'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

const createHoc = (hook) => (map) => (component) => {
    const hoc = (props) => {
        const [state, actions] = hook();
        return react.createElement(component, Object.assign({}, props, map(state, actions)));
    };
    hoc.displayName = `WithDakpan(${component.displayName || component.name || 'Unknown'})`;
    return hoc;
};

const createHook = (context, actions) => () => {
    const state = react.useContext(context);
    return [
        state,
        actions
    ];
};

const createProvider = (context, initialState, callback) => ({ children }) => {
    const [value, setValue] = react.useState(initialState);
    callback(() => value, setValue);
    react.useEffect(() => callback, []);
    return react.createElement(context.Provider, {
        value
    }, children);
};

const createDakpan = (initialState) => (actions) => {
    const context = react.createContext(typeof initialState === 'object' ? initialState : initialState());
    let getState;
    let setState;
    const callback = (get, set) => {
        getState = get;
        setState = set;
    };
    const mapped = Object.keys(actions).reduce((previous, current) => {
        const execute = (...args) => () => Promise.resolve(actions[current](...args)(getState()))
            .then((state) => {
            if (state && setState) {
                setState(state);
            }
        });
        const action = (...args) => execute(...args)();
        action.c = execute;
        return Object.assign({}, previous, { [current]: action });
    }, {});
    const hook = createHook(context, mapped);
    return [
        createProvider(context, initialState, callback),
        hook,
        createHoc(hook)
    ];
};

exports.createDakpan = createDakpan;
//# sourceMappingURL=index.js.map
